---
# This is example values file for the template
# This is a YAML-formatted file.
# Declare variables to be passed into your templates.

image:
  repository: "docker.io/batfish/allinone"
  tag: "latest"

service:
  - node_port: 8888
    container_port: 8888
    name: "jupyter-notebook"
  - node_port: 9997
    container_port: 9997
    name: "java1"
  - node_port: 9996
    container_port: 9996
    name: "java2"
