---
api_version: "v1"
name: "{{ data.map_name }}"
description: "{{ data.map_name }} deployment"

# This is the map version. This version number should be incremented each time you make changes
# to the map and its templates, including the app version.
# Versions are expected to follow Semantic Versioning (https://semver.org/)
version: "0.1.0"

# This is the version number of the application being deployed. This version number should be
# incremented each time you make changes to the application. Versions are not expected to
# follow Semantic Versioning. They should reflect the version the application is using.
# It is recommended to use it with quotes.
app_version: "0.1.0"
